# - Config file for the VecGeom package
# Defines the following variables
#   VECGEOM_FOUND        - Found the VECGEOM library
#   VECGEOM_INCLUDE_DIR  - include directories for VecGeom
#   VECGEOM_LIBRARIES    - libraries to link against
#   USOLIDS_INCLUDE_DIRS - include directories for USolids
#   USOLIDS_LIBRARIES    - libraries to link against
#   VECGEOM_DEFINITIONS - some core flags used to build the vecgeom library and which need to be reused 
#                               in projects using VecGeom 
#   VECGEOM_INSTALL_DIR     - the top level installation path
#   VECGEOM_EXTERNAL_INCLUDES - external include paths used during vecgeom compilation (for automatic "discovery" of dependencies)
#   VECGEOM_C_COMPILER      - the C compiler used to compile VecGeom
#   VECGEOM_CXX_COMPILER    - the CXX compiler used to compile VecGeom

if(NOT VecGeom_FIND_QUIETLY)
  message(STATUS
    "Using VecGeom install configuration at ${CMAKE_CURRENT_LIST_FILE}.")
endif()

get_filename_component(THIS_DIR "${CMAKE_CURRENT_LIST_FILE}" PATH)
set(USOLIDS_INCLUDE_DIRS "${THIS_DIR}/../../../include")
set(USOLIDS_LIBRARIES "dl;rt;${THIS_DIR}/../..//libvecgeom.a;${THIS_DIR}/../..//libusolids.a")
set(VECGEOM_INCLUDE_DIR "${THIS_DIR}/../../../include")
set(VECGEOM_LIBRARIES "dl;rt;${THIS_DIR}/../..//libvecgeom.a;${THIS_DIR}/../..//libusolids.a")
set(VECGEOM_INSTALL_DIR "$ENV{VECGEOM_FQ_DIR}")
set(VECGEOM_COMPILE_OPTIONS  -fabi-version=0 -msse3)
set(VECGEOM_CUDA_STATIC_LIBRARY "")

set(VECGEOM_DEFINITIONS   -DVECGEOM_SCALAR -DVECGEOM_QUADRILATERALS_VC -DVECGEOM_NO_SPECIALIZATION -DVECGEOM_USOLIDS -DVECGEOM_INPLACE_TRANSFORMATIONS -DVECGEOM_USE_INDEXEDNAVSTATES)
add_definitions(${VECGEOM_DEFINITIONS})

# add the include directories used to compile VecGeom
set(VECGEOM_EXTERNAL_INCLUDES ";/home/vagrant/build/debug/build/vecgeom/v0_4a/build/Linux64bit+4.4-2.23-e14-debug/installExternals/VecCore-0.4.1$ENV{VECGEOM_FQ_DIR}/lib/cmake/VecCore/../../../include")

set(VECGEOM_C_COMPILER "/home/vagrant/build/debug/build/gcc/v6_3_0/Linux64bit+4.4-2.23/bin/gcc")
set(VECGEOM_CXX_COMPILER "/home/vagrant/build/debug/build/gcc/v6_3_0/Linux64bit+4.4-2.23/bin/g++")

set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS}  -fabi-version=0 -msse3")
if ("${CMAKE_CXX_COMPILER_ID}" STREQUAL "GNU")
  set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} ")
endif()
if (("${CMAKE_CXX_COMPILER_ID}" STREQUAL "AppleClang")
    OR ("${CMAKE_CXX_COMPILER_ID}" STREQUAL "Clang"))
  set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} ")
endif()
if ("${CMAKE_CXX_COMPILER_ID}" STREQUAL "Intel")
  set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} ")
endif()
set(VECGEOM_GEANT4 )
set(VECGEOM_CUDA_ARCH "")
set(VECGEOM_FOUND TRUE)

